<?php

namespace Drupal\search_api_fusion\Plugin\SolrConnector;

use Drupal\Core\Form\FormStateInterface;
use Drupal\search_api_solr\Plugin\SolrConnector\BasicAuthSolrCloudConnector;
use Solarium\Core\Client\Endpoint;
use Solarium\Core\Query\QueryInterface;
use Solarium\QueryType\Select\Query\Query;

/**
 * Fusion connector.
 *
 * @SolrConnector(
 *   id = "fusion",
 *   label = @Translation("Fusion"),
 *   description = @Translation("A connector for Lucidworks Fusion.")
 * )
 */
class FusionConnector extends BasicAuthSolrCloudConnector {

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'port' => '8764',
      'fusion_qprofile_search' => '',
    ] + parent::defaultConfiguration();
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);

    // Rename titles and descriptions to reference Fusion instead of Solr.
    $form['host']['#title'] = $this->t('Fusion host');
    $form['host']['#description'] = $this->t('The host name or IP of your Fusion server, e.g. localhost or www.example.com.');

    $form['port']['#title'] = $this->t('Fusion port');
    $form['port']['#description'] = $this->t('The standard Fusion port is 8764.');

    $form['core']['#title'] = $this->t('Fusion app');
    $form['core']['#description'] = $this->t('The name of the Fusion app.');

    // Set default context to /api/solr (Fusion's Solr API).
    $form['context'] = [
      '#type' => 'value',
      '#value' => '/api/solr',
    ];

    // Extra fields specific to Fusion.
    $form['fusion_qprofile_search'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Fusion search query profile'),
      '#description' => $this->t('The Fusion query profile to use when searching.'),
      '#required' => TRUE,
      '#default_value' => $this->configuration['fusion_qprofile_search'] ?? '',
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function pingServer() {
    // SolrConnectorPluginBase pings Fusion on admin/info/system which may not
    // exist. Ping Fusion on admin/ping instead.
    return $this->pingCore();
  }

  /**
   * {@inheritdoc}
   */
  public function reloadCore() {
    // Not applicable to Fusion, just return success.
    return TRUE;
  }

  /**
   * {@inheritdoc}
   *
   * Send autocomplete requests to the Query profile:
   *   /api/apps/<app>/query/<autocomplete-query-profile>
   *
   * Instead of the default:
   *   /api/solr/<app>/select
   */
  public function search(Query $query, ?Endpoint $endpoint = NULL) {
    return $this->queryProfile($query, $this->configuration['fusion_qprofile_search']);
  }

  /**
   * Send request to a Fusion query profile.
   *
   * @param \Solarium\Core\Query\QueryInterface $query
   *   Query to send to the the query profile.
   * @param string $query_profile
   *   ID of Fusion query profile.
   *
   * @return \Solarium\Core\Client\Response
   *   The Solarium response object.
   *
   * @see https://doc.lucidworks.com/fusion/5.5/361/query-api
   */
  public function queryProfile(QueryInterface $query, $query_profile) {
    $this->connect();

    // Set context to /api/apps (instead of the default /api/solr which is set
    // in the admin form).
    $apps_endpoint = $this->createEndpoint('search_api_fusion_apps', ['context' => 'api/apps']);

    // Set handler to /query/<query-profile>.
    $query->setHandler('query/' . $query_profile);

    // Send request and return response.
    $request = $this->solr->createRequest($query);
    return $this->executeRequest($request, $apps_endpoint);
  }

}
