<?php

namespace Drupal\search_api_fusion\Plugin\views\area;

use Drupal\Core\Url;
use Drupal\Core\Link;
use Drupal\views\Plugin\views\area\AreaPluginBase;

/**
 * Provides an area for spellcheck suggestions.
 *
 * @ViewsArea("search_api_fusion_spellcheck")
 *
 * @see Drupal\search_api_spellcheck\Plugin\views\area\DidYouMeanSpellCheck
 */
class FusionSpellcheck extends AreaPluginBase {

  /**
   * The current query parameters.
   *
   * @var array
   */
  protected $currentQuery;

  /**
   * Render the area.
   *
   * @param bool $empty
   *   (optional) Indicator if view result is empty or not. Defaults to FALSE.
   *
   * @return array
   *   In any case we need a valid Drupal render array to return.
   */
  public function render($empty = FALSE) {
    /** @var Drupal\search_api\Plugin\views\query\SearchApiQuery */
    $query = $this->query;
    /** @var ResultSetInterface $result */
    $results = $query->getSearchApiResults();

    // Return nothing if Fusion provided no spellcheck suggestion.
    $response = $results->getExtraData('search_api_solr_response');
    if (empty($response['spellcheck']['suggestions'])) {
      return [];
    }

    // If Fusion provided spellcheck suggestions: order those by frequency.
    $suggestions = $response['spellcheck']['suggestions'];
    foreach ($suggestions as $word => $data) {
      if (isset($data['suggestion'])) {
        usort($suggestions[$word]['suggestion'],
            function ($suggestion1, $suggestion2) {
              return $suggestion1['freq'] < $suggestion2['freq'];
            });
      }
    }

    // Extract suggestion with highest frequency.
    reset($suggestions[$word]['suggestion']);
    $best_suggestion = current($suggestions[$word]['suggestion']);

    // Return hyperlink "Did you mean: <best spellcheck suggestion> ?".
    return [
      '#theme' => 'search_api_fusion_spellcheck',
      '#label' => $this->t('Did you mean:'),
      '#link' => $this->getSuggestionLink($best_suggestion['word']),
    ];
  }

  /**
   * Gets the suggestion link.
   *
   * @param string $suggestion
   *   The suggested search query.
   *
   * @return \Drupal\Core\Link
   *   The suggestion link.
   */
  protected function getSuggestionLink($suggestion) {
    if ($this->view->hasUrl()) {
      $path = '/' . $this->view->getPath();
    }
    else {
      $path = \Drupal::service('path.current')->getPath();
    }

    $currentQuery = $this->getCurrentQuery();

    unset($currentQuery['q']);
    $url = Url::fromUserInput($path, [
      'query' => [
        'query' => $suggestion,
      ] + $currentQuery,
    ]);

    return Link::fromTextAndUrl($suggestion, $url);
  }

  /**
   * Gets the current query parameters.
   *
   * @return array
   *   Key value of parameters.
   */
  protected function getCurrentQuery() {
    if (NULL === $this->currentQuery) {
      $this->currentQuery = \Drupal::request()->query->all();
    }
    return $this->currentQuery;
  }

}
